(**************************************************************************)
(*                                                                        *)
(*  Copyright (C) 2001-2003,                                              *)
(*   George C. Necula    <necula@cs.berkeley.edu>                         *)
(*   Scott McPeak        <smcpeak@cs.berkeley.edu>                        *)
(*   Wes Weimer          <weimer@cs.berkeley.edu>                         *)
(*   Ben Liblit          <liblit@cs.berkeley.edu>                         *)
(*  All rights reserved.                                                  *)
(*                                                                        *)
(*  Redistribution and use in source and binary forms, with or without    *)
(*  modification, are permitted provided that the following conditions    *)
(*  are met:                                                              *)
(*                                                                        *)
(*  1. Redistributions of source code must retain the above copyright     *)
(*  notice, this list of conditions and the following disclaimer.         *)
(*                                                                        *)
(*  2. Redistributions in binary form must reproduce the above copyright  *)
(*  notice, this list of conditions and the following disclaimer in the   *)
(*  documentation and/or other materials provided with the distribution.  *)
(*                                                                        *)
(*  3. The names of the contributors may not be used to endorse or        *)
(*  promote products derived from this software without specific prior    *)
(*  written permission.                                                   *)
(*                                                                        *)
(*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *)
(*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *)
(*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS     *)
(*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *)
(*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *)
(*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *)
(*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;      *)
(*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER      *)
(*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT    *)
(*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN     *)
(*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE       *)
(*  POSSIBILITY OF SUCH DAMAGE.                                           *)
(*                                                                        *)
(*  File modified by CEA (Commissariat  l'nergie Atomique).             *)
(**************************************************************************)

(* $Id: cil_computation.mli,v 1.3 2008/11/18 12:13:40 uid568 Exp $ *)

(** Functors for building computations which use kernel datatypes. 
    @plugin development guide *)

module StmtSetRef(Info:Signature.NAME_DPDS) : 
  Computation.SET_REF_OUTPUT with type elt = Cil_types.stmt

module IntHashtbl(Data:Project.Datatype.S)(Info:Signature.NAME_SIZE_DPDS) :
  Computation.HASHTBL_OUTPUT with type key = int and type data = Data.t

module InstrHashtbl(Data:Project.Datatype.S)(Info:Signature.NAME_SIZE_DPDS) :
  Computation.HASHTBL_OUTPUT 
  with type key = Cil_types.kinstr and type data = Data.t

(** @plugin development guide *)
module StmtHashtbl(Data:Project.Datatype.S)(Info:Signature.NAME_SIZE_DPDS) :
  Computation.HASHTBL_OUTPUT 
  with type key = Cil_types.stmt and type data = Data.t

module VarinfoHashtbl(Data:Project.Datatype.S)(Info:Signature.NAME_SIZE_DPDS) :
  Computation.HASHTBL_OUTPUT 
  with type key = Cil_types.varinfo and type data = Data.t

module CodeAnnotationHashtbl
  (Data:Project.Datatype.S)(Info:Signature.NAME_SIZE_DPDS) :
  Computation.HASHTBL_OUTPUT 
  with type key = Cil_types.code_annotation and type data = Data.t

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
